package cn.iocoder.yudao.module.ecw.controller.admin.node;

import cn.hutool.core.collection.CollectionUtil;
import cn.iocoder.yudao.module.system.api.user.AdminUserApi;
import cn.iocoder.yudao.module.system.api.user.dto.AdminUserRespDTO;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import org.springframework.validation.annotation.Validated;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.*;

import javax.validation.*;
import javax.servlet.http.*;
import java.util.*;
import java.io.IOException;
import java.util.stream.Collectors;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

import cn.iocoder.yudao.framework.excel.util.ExcelUtils;

import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;

import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.*;

import cn.iocoder.yudao.module.ecw.controller.admin.node.vo.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.node.NodeDO;
import cn.iocoder.yudao.module.ecw.convert.node.NodeConvert;
import cn.iocoder.yudao.module.ecw.service.node.NodeService;

@Api(tags = "管理后台 - 服务网点")
@RestController
@RequestMapping("/ecw/node")
@Validated
public class NodeController {

    @Resource
    private NodeService nodeService;
    @Resource
    private AdminUserApi adminUserApi;

    @PostMapping("/create")
    @ApiOperation("创建服务网点")
    @PreAuthorize("@ss.hasPermission('ecw:node:create')")
    public CommonResult<Long> createNode(@Valid @RequestBody NodeCreateReqVO createReqVO) {
        return success(nodeService.createNode(createReqVO));
    }

    @PutMapping("/update")
    @ApiOperation("更新服务网点")
    @PreAuthorize("@ss.hasPermission('ecw:node:update')")
    public CommonResult<Boolean> updateNode(@Valid @RequestBody NodeUpdateReqVO updateReqVO) {
        nodeService.updateNode(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @ApiOperation("删除服务网点")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
    @PreAuthorize("@ss.hasPermission('ecw:node:delete')")
    public CommonResult<Boolean> deleteNode(@RequestParam("id") Long id) {
        nodeService.deleteNode(id);
        return success(true);
    }

    @GetMapping("/get")
    @ApiOperation("获得服务网点")
    @ApiImplicitParam(name = "id", value = "编号", required = true, example = "1024", dataTypeClass = Long.class)
//    @PreAuthorize("@ss.hasPermission('ecw:node:query')")
    public CommonResult<NodeRespVO> getNode(@RequestParam("id") Long id) {
        NodeDO node = nodeService.getNode(id);
        matchUser(Collections.singletonList(node));
        return success(NodeConvert.INSTANCE.convert(node));
    }

    @GetMapping("/list")
    @ApiOperation("获得服务网点列表")
//    @PreAuthorize("@ss.hasPermission('ecw:node:query')")
    public CommonResult<List<NodeRespVO>> getNodeList(NodeExportReqVO req) {
        List<NodeDO> list = nodeService.getNodeList(req);
        matchUser(list);
        return success(NodeConvert.INSTANCE.convertList(list));
    }

    @GetMapping("/list-all-simple")
    @ApiOperation(value = "获得服务网点精简信息列表", notes = "主要用于前端的下拉选项")
    public CommonResult<List<NodeSimpleRespVO>> getNodeListAllSimple() {
        List<NodeDO> list = nodeService.getNodeList();
        return success(NodeConvert.INSTANCE.convertListSimple(list));
    }

    @GetMapping("/page")
    @ApiOperation("获得服务网点分页")
//    @PreAuthorize("@ss.hasPermission('ecw:node:query')")
    public CommonResult<PageResult<NodeRespVO>> getNodePage(@Valid NodePageReqVO pageVO) {
        PageResult<NodeDO> pageResult = nodeService.getNodePageList(pageVO);
        List<NodeDO> nodeList = pageResult.getList();
        matchUser(nodeList);
        return success(NodeConvert.INSTANCE.convertPage(pageResult));
    }

    @GetMapping("/export-excel")
    @ApiOperation("导出服务网点 Excel")
    @PreAuthorize("@ss.hasPermission('ecw:node:export')")
    @OperateLog(type = EXPORT)
    public void exportNodeExcel(@Valid NodeExportReqVO exportReqVO,
                                HttpServletResponse response) throws IOException {
        List<NodeDO> list = nodeService.getNodeList(exportReqVO);

        matchUser(list);

        // 导出 Excel
        List<NodeExcelVO> datas = NodeConvert.INSTANCE.convertList02(list);
        ExcelUtils.write(response, "服务网点.xls", "数据", NodeExcelVO.class, datas);
    }

    private void matchUser(List<NodeDO> nodeList) {
        List<Long> userIdList = nodeList.stream()
                .filter(t -> t.getAdminId() != null)
                .map(NodeDO::getAdminId)
                .collect(Collectors.toList());
        if (CollectionUtil.isEmpty(userIdList)) return;
        List<AdminUserRespDTO> userList = adminUserApi.getUsers(userIdList);
        Map<Long, String> userNameMap = userList.stream()
                .collect(Collectors.toMap(AdminUserRespDTO::getId, AdminUserRespDTO::getNickname, (v1, v2) -> v2));
        nodeList.forEach(t -> {
            Long adminId = t.getAdminId();
            if (adminId != null && userNameMap.containsKey(adminId)) {
                t.setAdminName(userNameMap.get(adminId));
            }
        });
    }

}
