package cn.iocoder.yudao.module.ecw.service.node;

import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import java.util.*;
import cn.iocoder.yudao.module.ecw.controller.admin.node.vo.*;
import cn.iocoder.yudao.module.ecw.dal.dataobject.node.NodeDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;

import cn.iocoder.yudao.module.ecw.convert.node.NodeConvert;
import cn.iocoder.yudao.module.ecw.dal.mysql.node.NodeMapper;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.ecw.enums.ErrorCodeConstants.*;

/**
 * 服务网点 Service 实现类
 *
 * @author wlh
 */
@Service
@Validated
public class NodeServiceImpl implements NodeService {

    @Resource
    private NodeMapper nodeMapper;

    @Override
    public Long createNode(NodeCreateReqVO createReqVO) {
        // 插入
        NodeDO node = NodeConvert.INSTANCE.convert(createReqVO);
        nodeMapper.insert(node);
        // 返回
        return node.getId();
    }

    @Override
    public void updateNode(NodeUpdateReqVO updateReqVO) {
        // 校验存在
        this.validateNodeExists(updateReqVO.getId());
        // 更新
        NodeDO updateObj = NodeConvert.INSTANCE.convert(updateReqVO);
        nodeMapper.updateById(updateObj);
    }

    @Override
    public void deleteNode(Long id) {
        // 校验存在
        this.validateNodeExists(id);
        // 删除
        nodeMapper.deleteById(id);
    }

    private void validateNodeExists(Long id) {
        if (nodeMapper.selectById(id) == null) {
            throw exception(NODE_NOT_EXISTS);
        }
    }

    @Override
    public NodeDO getNode(Long id) {
        return nodeMapper.selectById(id);
    }

    @Override
    public List<NodeDO> getNodeList(Collection<Long> ids) {
        return nodeMapper.selectBatchIds(ids);
    }

    @Override
    public List<NodeDO> getNodeList() {
        return nodeMapper.selectList();
    }

    @Override
    public PageResult<NodeDO> getNodePage(NodePageReqVO pageReqVO) {
        return nodeMapper.selectPage(pageReqVO);
    }

    @Override
    public List<NodeDO> getNodeList(NodeExportReqVO exportReqVO) {
        return nodeMapper.selectList(exportReqVO);
    }

    @Override
    public PageResult<NodeDO> getNodePageList(NodePageReqVO pageReqVO) {
        IPage<NodeDO> mpPage = MyBatisUtils.buildPage(pageReqVO);
        mpPage = nodeMapper.queryPageList(mpPage, pageReqVO.getCityName(), pageReqVO.getKeywords());
        PageResult<NodeDO> pageResult = new PageResult<NodeDO>();

        pageResult.setList( mpPage.getRecords() );
        pageResult.setTotal( mpPage.getTotal() );
        pageResult.setRows( mpPage.getSize() );
        pageResult.setPage( mpPage.getCurrent() );
        pageResult.setPages( mpPage.getPages() );
        return pageResult;
    }
}
